;  _  -**-  _
; (_|-|/\()_\   chaosmon.s: Monitor program
;     -**-
; 
; This is the first application program for CHAOS. It's based on the old "chaos.s" downloader/monitor
; program.

		INCL z80:chaos/chaos.i

; Ideas:				disassembler
;						single-step and breakpoints (modify an RST xx to jump to this)

startaddr:	EQU	4000h		; Where we start up pointing to

; --- "Here goes another."

        ORG appbase			; Start of application.

		LD A,(os_mode)		; Check to see if CHAOS is safely running before we start ourself.
							; This is recommended code to include at the start of any CHAOS app.
		CP M_chaos
		JR Z,app_start
hang:	JR hang
app_start:

; --- Begin user application code...

monitor:
        LD IX,welcome
        CALL i_pstring

        LD HL,startaddr   ; HL contains the current address.

gloop:  LD IX,prompt
        CALL i_pstring
		LD IX,strbuf
		CALL i_gstring
		LD A,(strbuf)	; First character is the command
        CP 'd'
        JR NZ,notd

        ; 'd' command: display block
        LD A,L
        AND 11111000b   ; Set the low 8 bits to 0.
        LD L,A
        LD E,4          ; E-line counter (was 8 lines, now 4)
dline:  CALL outadr
        LD IX,colon
        CALL i_pstring
        PUSH HL
        LD C,8          ; C-pos counter  (8 positions)
ddat:   LD A,(HL)
        CALL outhex
        CALL outsp
        INC HL
        DEC C
        JR NZ,ddat
        POP HL
        LD C,8          ; C-pos counter  (8 positions)
ddat2:  LD A,(HL)
        CP 32           ; Is it 32 or more? (i.e. printable)
        JR NC,ddat3     ; Yep -> ddat3
ddat5:  LD A,'.'        ; No, so display a dot instead.
		JR ddat4
ddat3:  CP 128			; Is it 128 or more? not printable -> ddat5
		JR NC,ddat5
ddat4:	CALL i_out
        INC HL
        DEC C
        JR NZ,ddat2
		CALL outcr
        DEC E
        JR NZ,dline
        JP gloop

notd:   CP 'a'
        JR NZ,nota
        ; 'a' command: set address
		LD IX,strbuf+2
        CALL i_gethex
        LD H,A
		LD IX,strbuf+4
        CALL i_gethex
        LD L,A
        JP gloop

nota:   CP 'R'
        JR NZ,notr

        ; 'R' command: reboot.
        RST 00h
        ; We'll never get here.

notr:   CP 'G'
        JR NZ,notg

        ; 'G' command: run from this address
        JP (HL)
        ; We'll never get here.

notg:   CP 'h'
        JR NZ,noth

        ; 'h' command: help
        LD IX,help
        CALL i_pstring
        JP gloop

noth:   CP 'm'
        JR NZ,notm

        ; 'm' command: input
		LD IX,strbuf+2
        CALL i_gethex
        LD D,A
        LD A,D
        LD (HL),A
        CALL outadr
        LD IX,colon
        CALL i_pstring
        LD A,(HL)
        CALL outhex
        INC HL
        CALL outcr
        JP gloop

notm:   CP 'r'
        JR NZ,notrr
        ; 'r' - receive data
        LD IX,nimp
        CALL i_pstring
        JP gloop

notrr:  CP 'i'
        JR NZ,noti
        ; 'i' - in from a port
		LD IX,strbuf+2
        CALL i_gethex
        LD C,A
        IN A,(C)
        LD C,A
        CALL outcr
        LD A,C
        CALL outhex
		CALL outcr
        JP gloop

noti:   CP 'o'
        JR NZ,noto
        ; 'o' - out port, value
		LD IX,strbuf+2
        CALL i_gethex
        LD C,A
		LD IX,strbuf+5
        CALL i_gethex
        OUT (C),A
        JP gloop

noto:   CP 'c'
        JR NZ,notc
        ; 'c' - display ASCII code
		LD IX,hitakey
        CALL i_pstring
        CALL i_in
        LD C,A
        CALL i_out
        CALL outsp
        LD A,C
        CALL outhex
		CALL outcr
        JP gloop

notc:	CP 'x'
		JR NZ,notx
		; 'x' - exit the monitor
		RET
	
notx:	CP 'l'
		JR NZ,notl
		LD IX,strbuf+2
		PUSH HL
		CALL i_fread
		POP HL
		LD A,B
		CALL outhex
		LD A,C
		CALL outhex
		LD IX,bread
		CALL i_pstring
		JP gloop

; w 0123 filename
notl:	CP 'w'
		JR NZ,notw
		LD IX,strbuf+2
		CALL i_gethex
		LD B,A
		LD IX,strbuf+4
		CALL i_gethex
		LD C,A	
		LD IX,strbuf+7	
		PUSH HL
		LD A,'x'		; def type executable - not ideal.
		CALL i_fwrite
		POP HL
		JP gloop

notw:

nocmd:  LD IX,error
        CALL i_pstring
        JP gloop

; --- Routines

outadr: LD A,H
        CALL outhex
        LD A,L
        CALL outhex
        RET

outsp:	PUSH AF
		LD A,' '
		CALL i_out
		POP AF
		RET

outcr:	PUSH AF
		LD A,13
		CALL i_out
		LD A,10
		CALL i_out
		POP AF
		RET

outhex:	PUSH AF
		PUSH IX
		LD IX,hexbuf
		CALL i_fhex
		LD IX,hexbuf
		CALL i_pstring
		POP IX
		POP AF
		RET

; --- Data area

hexbuf: DB "xx",eos
strbuf: DS 80
		DB eos
welcome: DB "Mon 1.3 OK, h=help",cr,lf,eos
prompt: DB "# ",eos
error:  DB "Unknown command!",cr,lf,eos
nimp:   DB "Not implemented yet.",cr,lf,eos
bread:	DB " bytes read.",cr,lf,eos
colon:  DB ": ",eos
help:   DB cr,lf
		DB "Mon: Addr Val Port File Len",cr,lf
        DB "a <a>     set addr",cr,lf
        DB "d         dump block",cr,lf
        DB "R         restart 00h",cr,lf
        DB "G         go",cr,lf
        DB "h         help",cr,lf
        DB "m <v>     input data",cr,lf
        DB "r         receive data",cr,lf
        DB "i <p>     input from port",cr,lf
        DB "o <p> <v> output to port",cr,lf
        DB "c         display ASCII code",cr,lf
		DB "l <f>     load file to current address",cr,lf
		DB "w <l> <f> write current address to file",cr,lf
		DB "x         exit",cr,lf
        DB eos
hitakey: DB "Hit a key...",cr,lf,eos
		END
