; chaos.i - CHAOS application values and OS constants

; Important system addresses.
sysbase: EQU 0100h		; The address of the i_X jump table.
basepage: EQU 0010h		; The base page - useful system variables.
appbase: EQU 1800h		; 
lcdbuf:	EQU 003bh		; The buffer for a small display. 

; The system routines.
i_init:     EQU     sysbase
i_wboot:    EQU     sysbase+03h
i_status:   EQU     sysbase+06h
i_in:       EQU     sysbase+09h
i_out:      EQU     sysbase+0Ch
i_pstring:  EQU     sysbase+0Fh
i_gstring:  EQU     sysbase+12h
i_locate:   EQU     sysbase+15h
i_strcmp:   EQU     sysbase+18h
i_strncmp:  EQU     sysbase+1Bh
i_strlen:   EQU     sysbase+1Eh
i_strucase: EQU     sysbase+21h
i_strcopy:  EQU     sysbase+24h
i_setdevu:  EQU     sysbase+27h
i_getdevu:  EQU     sysbase+2Ah
i_fhex:		EQU		sysbase+2Dh
i_gethex:	EQU		sysbase+30h
i_beep:		EQU		sysbase+33h
i_setflags:	EQU		sysbase+36h
i_getflags:	EQU		sysbase+39h
i_getrandom: 	EQU		sysbase+3Ch
i_fread:	EQU		sysbase+3Fh
i_fwrite:	EQU		sysbase+42h
i_dreset:	EQU		sysbase+45h
i_dnext:	EQU		sysbase+48h
i_fsize:	EQU		sysbase+4Bh
i_ftype:	EQU		sysbase+4Eh
i_fdel:		EQU		sysbase+51h
i_fsfree:	EQU		sysbase+54h

; Base page addresses. In general, these should not be directly modified by the user's program. Go through
; the operating system.
start:		EQU		basepage
system:		EQU		basepage+03h 	; A routine. Obsolete now. Could be used for "temporary call" code.
device:     EQU     basepage+06h
unit:       EQU     basepage+07h
disk:       EQU     basepage+08h 
track:      EQU     basepage+09h 
os_flags:	EQU		basepage+0Ah	; This is set and read by the setflags and getflags routines. It is
									; not recommended that user programs change this directly. 
; _0	sound muted 
os_mode:	EQU		basepage+0Bh	; This can be checked by application programs and interrupts to find
									; out if CHAOS is running or not.
; $0	Nothing running - quit		M_nil
; $42	CHAOS running normally		M_chaos
; $BD	Boot downloader running		M_bootstrap
random:		EQU		basepage+0Ch	; A pseudo-random byte, incremented by various routines. It is deliberately
									; NOT set on startup. One rather nice spinoff of this is that it therefore
									; gets saved from session to session.
ccp_prompt:	EQU		basepage+0Dh	; The prompt character used by the CCP.
dirptr:		EQU		basepage+0Eh	; The dir pointer (low byte)
dirptrh:	EQU		basepage+0Fh	;                 (high byte)
v_xcoord:	EQU		basepage+10h	; X coord on text display (when video is used)
v_ycoord:	EQU		basepage+11h	; Y coord

; Character values.
eos:        EQU     0           
cr:         EQU     13          ; Return.
lf:         EQU     10
fieldsep:	EQU		1			; Non-printable, but not the same as eos.

; Routine constants, and special character values.
K_no_key:	EQU 	0FFh		; "no key pressed" rawcode
K_bad_key:	EQU		0FEh		; "unknown key pressed" rawcode
K_deb_lenA:	EQU		0F0h		; length of debounce delay on keydown		F0h
K_deb_lenB:	EQU		60h			; length of debounce delay on keyup			60h
K_shift:	EQU		0h			; shift sushicode
G_hres:		EQU		5h			; pixels per line
S_defdev:	EQU		0h			; default device and unit		0=con 1=ser
S_defun:	EQU		0h			;								(ignored for now)
M_nil:		EQU		0h			; Nothing is running - CHAOS has quit.
M_chaos:	EQU		42h			; CHAOS is running.
M_bootstrap: EQU	0BDh		; The boot downloader is running.

