;  _  -**-  _
; (_|-|/\()_\   The CHAOS operating system kernel and CCP. Adam Sampson, 1997-8
;     -**-
;
	; The length of image must NOT exceed 6144 for programs to work.
	; (At least in an 8K system.)
	
; CHAOS -- an OS for Z80 systems.		*asl/c/UNIX source code*

		cpu Z80

; The name of the machine this is being compiled for.
machine:	equ	"sim"
		
; Include the system calls and addresses.
		include chaos.i
	
; A few values that application programs don't need to know.
bootbase: 	equ 0h		; Where this boots from.
sysstack:	equ 00FEh	; Just below the sysbase (needs two bytes)

; This is the soft-start point -- probably at 0h.
		ORG bootbase	; 0x00

		DI		; don't want to do interrupts
		JP osbase	

; If your machine needs dummy interrupt handlers, then uncomment this lot.
; It takes quite a bit of RAM.

;		ORG 8h
; Interrupt points - NMI=0x66, INTi=(i+1)*8, IM1=0x38 (int7)
;int1:		JP handleint7	; 0x08
;		DS 5
;int2:		JP handleint7	; 0x10
;		DS 5
;int3:		JP handleint7	; 0x18
;		DS 5
;int4:		JP handleint7	; 0x20
;		DS 5
;int5:		JP handleint7	; 0x28
;		DS 5
;int6:		JP handleint7	; 0x30
;		DS 5
;int7:		JP handleint7	; 0x38	
;		DS 43		; 0x3b is the LCD buffer.
;nmi:		JP handlenmi	; 0x66

;;; The table of BIOS calls.

		ORG sysbase
		JP s_init
		JP s_wboot
		JP s_status
		JP s_in
		JP s_out
		JP s_pstring
		JP s_gstring
		JP s_locate
		JP s_strcmp
		JP s_strncmp
		JP s_strlen
		JP s_strucase
		JP s_strcopy
		JP s_setdevu
		JP s_getdevu
		JP s_fhex
		JP s_gethex
		JP s_beep		; 33
		JP s_setflags		; 36
		JP s_getflags		; 39
		JP s_getrandom		; 3c
		JP s_fread		; 3f
		JP s_fwrite		; 42
		JP s_dreset		; 45
		JP s_dnext		; 48
		JP s_fsize		; 4B
		JP s_ftype		; 4E
		JP s_fdel		; 51
		JP s_fsfree		; 54

; Now the interrupt handlers.
handleint7:	EI
		RETI

handlenmi:	RETN

; This is the entry point for the OS. The bootloader jumps to here.

osbase:		JP i_init	; BIOS cold-boot call. Loads the CCP.

; CCCP: the CHAOS Console Command Processor.

ccp:		LD IX,welcome   ; Display the welcome message.
		CALL i_pstring            

ccpentry:	LD B,20h	; Bip!
		LD C,1Fh
		CALL i_beep

; NB: these routines are very dependant on the behaviour of the C register in string routines.

ccpmain:	LD A,(ccp_prompt)
		CALL i_out
		LD IX,strbuf    ; Get a string from the user.
		CALL i_gstring

		LD IX,strbuf	; Check the length - don't do anything if zero.
		CALL i_strlen
		OR A
		JP Z,ccpmain

		LD IY,ccp_commands
ccp_cmd0:	LD A,(IY+0)	; Get past the fieldsep.
		INC IY
		CP fieldsep
		JR NZ,ccp_cmd0
ccp_cmd1:	LD A,(IY+1)	; Note the reversal of order.
		LD (ccp_jump+1),A
		LD A,(IY+0)
		LD (ccp_jump+2),A
		INC IY
		INC IY
; Address now in the ccp_jump routine; calling it will go to the 
; current thing being pointed at. IY points to the string.
		LD A,(IY+0)
		CP 0FFh
		JR Z,ccp_nocmdfound
		LD IX,strbuf	; Get length of input string.
		CALL i_strlen
		LD C,A
		PUSH BC		; Store it...
		PUSH IY
		POP IX
		CALL i_strlen	; IY not altered.
		POP BC		; ... and get it back.
		CP C		; Is it the same as the command?
		JR Z,ccp_cmd2	
; If the lengths of the input string and the command string that was
; just found are equal, then go to ccp_cmd2.
; They're not - we need to check for the space.
		LD C,A		; Load C with the length of the command string.
		LD IX,strbuf
		CALL i_strncmp	; Compare C characters of the strings.
		JR NZ,ccp_cmdnx ; not the same... bother.
		INC IX		; Now at the "space"
		LD A,(IX+0)
		CP 32		; Is it a space?
		JR NZ,ccp_cmdnx	; Nope.
		JR ccp_cmdyes	; YES! YES! YES!
ccp_cmd2:	LD IX,strbuf
		CALL i_strcmp
		JR NZ,ccp_cmdnx
				; fall through to...
ccp_cmdyes:	CALL ccp_jump	; Call the command's code.
		JP ccpmain
ccp_cmdnx:	JR ccp_cmd0	; check the next command on the list
	
; No command was found in the table... check the disk.
ccp_nocmdfound:
; Try and open the file with the command name given. In order to do this
; without losing arguments, we copy the name into a buffer up to the first
; space or null.
		LD IX,strbuf
		LD IY,ccp_cmdbuf
ccp_ncm1:	LD A,(IX+0)
		LD (IY+0),A
		OR A
		JR Z,ccp_ncm2
		CP 32
		JR Z,ccp_ncm2
		INC IX
		INC IY
		JR ccp_ncm1
ccp_ncm2:	XOR A
		LD (IY+0),A
		LD IX,ccp_cmdbuf
		CALL i_ftype
		CP 'x'
		JR NZ,ccp_nocmd
		LD IX,ccp_cmdbuf
		LD HL,appbase
		CALL i_fread
		LD A,B		; if zero length returned, not found
		OR C
		JR Z,ccp_nocmd
		LD IX,strbuf
		CALL appbase
		JP ccpmain


ccp_nocmd:	LD IX,badcmd
		CALL i_pstring
		JP ccpmain

ccpend:		LD IX,shutdown
		CALL i_pstring

		LD A,M_nil	; Set the mode; CHAOS might not work any more.
		LD (os_mode),A
			
		JP 2000h        ; return to the monitor, if there's one there,
				; or back into the OS under pocket80.

; Command table. 
; These are in the format fieldsep, <hi-adr>, <lo-adr>, <name-string>, eos.

ccp_commands:
		DB fieldsep,ccp_Cbye/256,ccp_Cbye&255,"bye",eos
		DB fieldsep,ccp_Chelp/256,ccp_Chelp&255,"help",eos
		DB fieldsep,ccp_Cmode/256,ccp_Cmode&255,"mode",eos
		DB fieldsep,0FFh,0FFh,0FFh,eos

; CCP commands.

ccp_Cbye:	JP ccpend	; doesn't return, ever

ccp_Chelp:	LD IX,helpmsg
		CALL i_pstring
		LD IX,ccp_commands
ccp_help0:	INC IX		; skip the fieldsep and address
		INC IX
		INC IX
		LD A,(IX+0)
		CP 0FFh
		RET Z
		CALL i_pstring
		CALL ccp_help1	; output the crlf
		INC IX
		JR ccp_help0

ccp_help1:	LD A,cr
		CALL i_out
		RET

helpmsg:	DB "Inbuilt cmds:", cr, eos

ccp_Cmode:	LD IX,strbuf
		CALL i_strlen
		CP 07h
		JR C,ccp_moded	; if too few characters, just display the mode.
		; Set the mode.
		LD IX,strbuf
		LD IY,ccp_modetab
		LD A,(IX+6)	; the mode name
		LD C,A
		LD B,1h
ccp_mode0:	LD A,(IY+0)
		CP C
		JR Z,ccp_mode1
		OR A
		JR Z,ccp_modebad
		RLC B
		INC IY
		JR ccp_mode0
ccp_mode1:	; B now contains mode mask, i.e. mode 0 = %00000001
		LD A,B
		CPL
		LD B,A
		CALL i_getflags
		AND B		; A=mode byte, with appropriate bit set to 0
		LD C,A		; store the new mode flags into C.
		LD A,(IX+5)	; load the +/- into A
		CP '+'
		JR Z,ccp_modes
		JR ccp_modev
ccp_modes:	LD A,B
		CPL
		OR C		; set the bit
		LD C,A
ccp_modev:	LD A,C
		CALL i_setflags
ccp_moded:	; Display the current mode flags.
		LD IX,modebyte
		CALL i_pstring
		CALL i_getflags
		LD IX,hexbuf
		CALL i_fhex
		LD IX,hexbuf
		CALL i_pstring
		RET
ccp_modebad:	LD IX,badmode
		CALL i_pstring
		RET

ccp_modetab:	DB "qxxxxxxx",eos
badmode:	DB "Unknown mode", cr, eos
modebyte:	DB "Mode byte:", eos

; Messages.

welcome:	DB "CHAOS 2.1 ready", cr, eos
shutdown:	DB "CCCP exiting...", cr, eos
badcmd:		DB "Nothing happens", cr, eos

; The routines themselves.

;;; "init" Initialise CHAOS for the first time.

s_init:		LD SP,sysstack

; Configure the devices.
		CALL ttyinit
		CALL rconinit
		CALL batinit
		CALL uc1init

; Set up the base page.
		LD A,S_defdev	; Set the current device.
		LD B,S_defun
		CALL i_setdevu
		XOR A		; Enable sound.
		CALL s_setflags
		
		LD A,'.'	; Set the prompt character
		LD (ccp_prompt),A

		LD A,M_chaos	; Set the mode; CHAOS is now up.
		LD (os_mode),A
			
		JP ccp

;;; "wboot" Warm boot - go back to the CCP.

s_wboot:	JP ccpentry

;;; "status" Current device status.

s_status:	LD A,(device)
		OR A
		JP Z,ttyst
	        CP 01h
	        JP Z,rconst
	        CP 02h
		JP Z,batst
		JP uc1st

;;; "in" Read a character from the current device.

s_in:		CALL rn_inc	; Give the random-number generator a shove.
		LD A,(device)
		OR A
		JP Z,ttyin
		CP 01h
		JP Z,rconin
		CP 02h
		JP Z,batin
		JP uc1in
	
;;; "out" Write a character to the current device.

s_out:		PUSH AF
		PUSH BC
		CALL s_metaout
		POP BC
		POP AF
		RET

s_metaout:	LD C,A
		LD A,(device)
		OR A
		JP Z,ttyout
		CP 01h
		JP Z,rconout
		CP 02h
		JP Z,batout
		JP uc1out

;;; "gstring" Get a string from the current device into IX.
s_gstring:	LD D,0h
bdrcb_lp:	CALL s_in
		PUSH AF
		CALL s_out
		POP AF
		CP 08h          ; Delete.
		JR NZ,bdrcb_tab
		XOR A		; Are there any characters to delete?
		CP D
		JR NZ,bdrcb_dl2
		LD A,'>'
		CALL s_out
		JR bdrcb_lp
bdrcb_dl2:	LD A,' '        ; bdconin has already echoed the backspace.
		CALL s_out
		LD A,08h
		CALL s_out
		DEC IX
		DEC D
		JR bdrcb_lp
bdrcb_tab:	CP 09h          ; Tab
		JR NZ,bdrcb_ret
				; Ignore -- should be expanded to 8x' '
bdrcb_ret:	CP cr           ; Return
		JR NZ,bdrcb_char
		LD A,eos
		LD (IX+0),A
		JR bdrcb_end
bdrcb_char:	LD (IX+0),A
		INC IX
		INC D
		JR bdrcb_lp
bdrcb_end:	RET

;;; "pstring" Print a string in IX to the current device.
	
s_pstring:	PUSH AF
s_ps_lp:  	LD A,(IX+0)
		CP eos
		JR Z,s_ps_ret
		CALL s_out
		INC IX
         	JR s_ps_lp
s_ps_ret: 	POP AF
		RET

;;; "locate" Move the cursor on the current device.
	
; This doesn't actually do anything yet. :(
s_locate:
		RET

;;; "strcmp" Compare two strings in IX and IY, and set Z if they're equal.
	
s_strcmp:	LD A,(IX+0)
		CP (IY+0)
		RET NZ
		CP eos
		RET Z
		INC IX
		INC IY
		JR s_strcmp
	
;;; "strncmp" Compare C characters of IX and IY. Set Z if they're equal.

s_strncmp:
		LD A,(IX+0)
		CP (IY+0)
		RET NZ
		CP eos
		RET Z
		LD A,1h
		CP C
		RET Z
		INC IX
		INC IY
		DEC C
		JR s_strncmp

;;; "strlen" Get the length of string IX into C.

s_strlen:	LD C,0h
strlen_lp:	LD A,(IX+0)
		CP eos
		JR Z,s_strlen1
		INC IX
		INC C
		JR strlen_lp
s_strlen1:	LD A,C
		RET

;;; "strucase" Make a string upper case, pointed to by IX.

s_strucase:	LD A,(IX+0)
		CP eos
		RET Z
		CP 61h
		JR C,ucase_cont
		CP 7Bh
		JR NC,ucase_cont
		AND 11011111b
ucase_cont:	LD (IX+0),A
		INC IX
		JR s_strucase

;;; "strcopy" Copy a string from IX to IY, up to eos.
	
s_strcopy:	LD A,(IX+0)
		LD (IY+0),A
		CP eos
		RET Z
		INC IX
		INC IY
		JR NZ,s_strcopy
			
;;; "setdevu" Set the current device and unit.

s_setdevu:	LD (device),A
		LD A,B
		LD (unit),A
		RET

;;; "getdevu" Get the current device and unit.
	
s_getdevu:	LD A,(unit)
		LD B,A   
		LD A,(device)
		RET

;;; "fhex" Format a two-digit hex number from A into IX.
	
s_fhex:		PUSH AF
		PUSH BC
		LD B,A
		AND 0F0h
		rlca
		rlca
		rlca
		rlca
		CALL s_fhexi
		LD A,B
		AND 0Fh
		CALL s_fhexi
		POP BC
		POP AF
		RET

s_fhexi:	PUSH AF
		PUSH BC
		PUSH HL
		LD B,0h
		LD C,A
		LD HL,s_hextab
		ADD HL,BC
		LD A,(HL)
		LD (IX+0),A
		INC IX
		POP HL
		POP BC
		POP AF
		RET

;;; "gethex" Get a two-digit hex number from string IX into A.
	

s_gethex:	PUSH BC
		LD A,(IX+0)
		CALL s_gethex1
		rlca
		rlca
		rlca
		rlca
		LD B,A
		LD A,(IX+1)
		CALL s_gethex1
		OR B
		POP BC
		RET

; Dehexify one digit (in A) and return its value in A.
s_gethex1:	PUSH BC
		PUSH IX
		LD C,A
		LD IX,s_hextab
		LD B,0h
s_gh1_0:	LD A,(IX+0)
		CP C
		JR Z,s_gh1_1
		OR A
		JR Z,s_gh1_2
		INC B
		INC IX
		JR s_gh1_0	
s_gh1_2:	LD B,0h
s_gh1_1:	LD A,B					
s_gh1end:	POP IX
		POP BC
		RET

s_hextab:	DB "0123456789abcdef",0h

;;; "beep" Make a beep.
	
s_beep:		LD A,(os_flags)
		AND 01h		; bit 0 = sound muted
		CP 01h
		RET Z
		JP soundbeep

;;; "setflags" Set the OS flags variable.
	
s_setflags:	LD (os_flags),A
		RET

;;; "getflags" Get the OS flags variable.
	
s_getflags:	LD A,(os_flags)
		RET

;;; "getrandom" Get a random 8-bit number.

;; This is not a good method. A better way would be to read R...
s_getrandom:	LD A,(random)
		RET

;;; "fread" Read a file from filename IX to HL, return length read in BC.

s_fread:	CALL s_internal_seek ; to get the base address of the file
		LD A,D
		OR E
		JR NZ,s_fread2	; if it wasn't found, return
		LD BC,0h
		RET
s_fread2:	PUSH IX
		PUSH DE		; temp. move the base adr into IX
		POP IX 
		LD C,(IX+0)	; and fetch the length into BC
		LD B,(IX+1)
		INC IX		; past size and type
		INC IX
		INC IX
s_fread1:	LD A,(IX+0)	; move DE past the \0
		INC IX
		OR A
		JR NZ,s_fread1  ; |
		PUSH IX
		POP DE		;...
		PUSH BC		; ldir destroys BC, HL, DE
		PUSH DE
		PUSH HL
		PUSH DE		; weird code to swap DE and HL
		PUSH HL		;  |
		POP DE		;  |
		POP HL		; done
		LDIR		; for(...;((DE++)=(HL++)) && BC--;BC>=0);
		POP HL		; restore BC, HL, DE
		POP DE
		POP BC
		POP IX
		RET

;;; "fwrite" Write a file from HL, length BC, type A to filename IX.

s_fwrite:	PUSH IX
		CALL s_internal_term
		PUSH IX
		POP IY		; IY contains the dest addr.
		POP IX
		LD (IY+0),C
		LD (IY+1),B
		LD (IY+2),A
		INC IY
		INC IY
		INC IY
s_fwrite_0:	LD A,(IX+0)	; write the filename in...
		LD (IY+0),A
		INC IX
		INC IY
		OR A
		JR NZ,s_fwrite_0
		PUSH IY
		PUSH BC
		PUSH IY		; DE<-dest address
		POP DE
		LDIR
		POP BC		; Now write the terminator entry
		POP IY
		ADD IY,BC
		XOR A
		LD (IY+0),A
		LD (IY+1),A
		LD (IY+2),A
		LD (IY+3),A
		RET

; Internal: return in DE the base address of the file with name IX,
; or 0h if not found.	
s_internal_seek:
		PUSH IX
		PUSH IY
		PUSH IX
		POP IY		; IY now has name of what we're looking for.
		PUSH BC
		LD BC,(dirptr)	; Store the dirptr
		CALL s_dreset	; Start looking at the start
s_int_nexttime:		
		CALL s_dnext
		LD A,(IX+0)	; hit end of dir?
		CP 0
		JR Z,s_int_nomore; yes, give up
		PUSH IX
		PUSH IY
		CALL s_strcmp	; are the names the same?
		POP IY
		POP IX
		JR NZ,s_int_nexttime ; nope, try again
		PUSH IX		; yes, DE=IX-3 to get base address
		POP DE
		DEC DE
		DEC DE
		DEC DE
		JR s_int_end	; and we're done
s_int_nomore:	LD DE,0h
s_int_end:	LD (dirptr),BC	; restore dirptr
		POP BC
		POP IY
		POP IX
		RET

;;; "dreset" Reset the directory pointer.
	
s_dreset:	PUSH BC
		LD BC,fsbase
		LD (dirptr),BC
		POP BC
		RET

;;; "dnext" Get the next file's name in IX.

s_dnext:	PUSH HL
		PUSH BC
		PUSH DE
		LD IX,(dirptr)
		LD E,(IX+0)	; DE contains the file length
		LD D,(IX+1)
		INC IX
		INC IX
		INC IX		; IX points at the current entry's name
		PUSH IX
		POP HL
s_dnext1:	LD A,(HL)	; Loop until HL points at the first data byte
		INC HL
		OR A
		JR NZ,s_dnext1
		ADD HL,DE	; HL now points at the next entry
		LD (dirptr),HL
		POP DE
		POP BC
		POP HL		; and restore HL
		RET

;;; "fsize" Get the size of file IX into BC.
	
s_fsize:	CALL s_internal_seek ; get the base address into DE
		LD A,D
		OR E
		JR Z,s_fsize_err
		PUSH DE
		POP IX
		LD B,(IX+1)
		LD C,(IX+0)
		RET
s_fsize_err:	LD BC,0h
		RET

;;; "ftype" Get the type of file IX into A.
	
s_ftype:	CALL s_internal_seek
		LD A,D
		OR E
		JR Z,s_ftype_err
		PUSH DE
		POP IX
		LD A,(IX+2)
		RET
s_ftype_err:	XOR A
		RET

;;; "fdel" Delete file IX.
	
s_fdel:		CALL s_internal_seek
		LD A,D
		OR E
		RET Z		; if not found, die silently
; DE now contains the base address of the file to be deleted,
; and is therefore the destination of the copy to be done.
; We now need to load HL with the base address of the next file.
		PUSH DE		; store until the end
		PUSH DE		; IX=DE
		POP IX
		LD L,(IX+0)
		LD H,(IX+1)
		INC IX		; get past size and type
		INC IX
		INC IX
s_fdel_5:	LD A,(IX+0)	; get past name
		INC IX
		OR A
		JR NZ,s_fdel_5
		PUSH IX
		POP DE
		ADD HL,DE	; HL now contains the base of the next file.
		PUSH HL
; Now we need to find the length. The easiest way is to find
; the end of the terminator file and then subtract (the current) HL.
		CALL s_internal_term
		PUSH IX
		POP BC		; BC now points at the base of the last file.
		INC BC			
		INC BC
		INC BC
		INC BC		; BC now points at last byte of terminator
		PUSH BC		; This is untidy and needs fixing
		PUSH HL
		POP BC
		POP HL
		SCF		; clear carry (set then complement)
		CCF
		SBC HL,BC	; HL now contains the length to be copied
		PUSH HL
		POP BC
		POP HL
		POP DE
		LDIR 		; (DE++)<-(HL++) BC--
		RET

; Internal: put the base address of the terminator file in IX.
s_internal_term:
		PUSH DE
		PUSH AF
		LD IX,fsbase
s_fdel_1:	LD A,(IX+3)	; have we got the end file yet?
		OR A		; yes-name first char is null
		JR Z,s_fdel_2
s_fdel_3:	LD E,(IX+0)	; nope... get the length
		LD D,(IX+1)	
		INC IX
		INC IX
		INC IX
s_fdel_4:	LD A,(IX+0)	; get past the name
		INC IX
		OR A
		JR NZ,s_fdel_4
		ADD IX,DE	; add len
		JR s_fdel_1
s_fdel_2:	POP AF
		POP DE
		RET

;;; "fsfree" Get the amount of space free in the filing system to HL.
	
; This relies on the memory map being |-chaos-|-fs-|-appbase-|
s_fsfree:	CALL s_internal_term
		PUSH IX
		POP BC
		LD HL,appbase
		SCF
		CCF
		SBC HL,BC 
dummy:		RET

; Selectively include device drivers to be compiled in to this kernel.
; This is where to add new device drivers.
		
		SWITCH machine
		CASE "a80"
			MESSAGE "AZZ80 drivers included."
			INCLUDE arch/a80/console.i
			INCLUDE arch/a80/serial.i
			INCLUDE arch/a80/sound.i
			INCLUDE arch/dummy/ucon.i
			INCLUDE arch/dummy/batch.i
		CASE "pocket"
			MESSAGE "Pocket80 drivers included."
			INCLUDE arch/pocket/console.i
			INCLUDE arch/dummy/serial.i
			INCLUDE arch/dummy/ucon.i
			INCLUDE arch/dummy/batch.i
			INCLUDE arch/dummy/sound.i
		CASE "sim"
			MESSAGE "Simulator drivers included."
			INCLUDE arch/sim/console.i
			INCLUDE arch/dummy/serial.i
			INCLUDE arch/dummy/ucon.i
			INCLUDE arch/dummy/batch.i
			INCLUDE arch/dummy/sound.i
		CASE "dummy"
			MESSAGE "All dummy drivers included."
			INCLUDE arch/dummy/console.i
			INCLUDE arch/dummy/serial.i
			INCLUDE arch/dummy/ucon.i
			INCLUDE arch/dummy/batch.i
			INCLUDE arch/dummy/sound.i
		ELSECASE
			MESSAGE "Caution: no drivers included!"
		ENDCASE
	
; *** Random number routines.

rn_inc:		PUSH AF
		LD A,(random)
		INC A
		LD (random),A
		POP AF
		RET

; *** Writable data area.

hexbuf:		DB "  x", cr, eos
ccp_jump:	JP ccpmain	; Used by the table routines in the CCP.
ccp_cmdbuf:	DB "xxxxxxxxxxxxxx",eos
strbuf:		DS 80		; 80 char buffer.
		DB eos		; Just to catch runaway print routines.
		DS 40				;  safety net
		DB eos		; to force p2bin to make the binary this long
fsbase:	
		END

